import java.util.Random;

public class Main {

	private Random random = new Random();

	// Local graph
	private Node[] nodes;
	// Local searcher
	private Searcher searcher = new SearcherImpl();

	/**
	 * Creates nodes of a graph.
	 * 
	 * @param nodeCount number of nodes
	 */
	public void createNodes(int nodeCount) {
		nodes = new Node[nodeCount];

		for (int i = 0; i < nodeCount; i++) {
			nodes[i] = new NodeImpl();
		}
	}

	/**
	 * Creates a fully connected graph.
	 */
	public void connectAllNodes() {
		for (int from = 0; from < nodes.length; from++) {
			for (int to = from + 1; to < nodes.length; to++) {
				nodes[from].addNeighbor(nodes[to]);
				nodes[to].addNeighbor(nodes[from]);
			}
		}
	}

	/**
	 * Creates a randomly connected graph.
	 * 
	 * @param edgeCount number of edges
	 */
	public void connectSomeNodes(int edgeCount) {
		for (int i = 0; i < edgeCount; i++) {
			final int from = random.nextInt(nodes.length);
			final int to = random.nextInt(nodes.length);

			nodes[from].addNeighbor(nodes[to]);
		}
	}

	/**
	 * Create a randomly connected graph and do a quick measurement.
	 * 
	 * @param nodeCount number of nodes
	 * @param edgeCount number of edges
	 * @param attemptCount number of measurements
	 */
	public void searchBenchmark(int nodeCount, int edgeCount, int attemptCount) {

		createNodes(nodeCount);
		connectSomeNodes(edgeCount);

		// Display measurement header.
		
		for (int i = 0; i < attemptCount; i++) {
			// Select two random nodes.
			final int idxFrom = random.nextInt(nodes.length);
			final int idxTo = random.nextInt(nodes.length);

			// Calculate distance, measure operation time
			final long startTimeNs = System.nanoTime();
			final int distance = searcher.getDistance(nodes[idxFrom], nodes[idxTo]);
			final long durationNs = System.nanoTime() - startTimeNs;

			// Calculate transitive distance, measure operation time
			final long startTimeTransitiveNs = System.nanoTime();
			final int distanceTransitive = searcher.getDistanceTransitive(4, nodes[idxFrom], nodes[idxTo]);
			final long durationTransitiveNs = System.nanoTime() - startTimeTransitiveNs;

			if (distance != distanceTransitive) {
				System.out.printf("Standard and transitive algorithms inconsistent (%d != %d)%n", distance,
						distanceTransitive);
			} else {
				// Print the measurement result.
				System.out.printf("%7d %7d %7d %8d %13d %13d%n", nodeCount, edgeCount, i, distance, durationNs / 1000, durationTransitiveNs / 1000);
			}
		}
	}

	public static void main(String[] args) {
		Main instance = new Main();
		instance.runAllBenchmarks();
	}
	private void runAllBenchmarks(){

		System.out.printf("%7s %7s %7s %8s %13s %13s%n", "Nodes", "Edges", "Attempt", "Distance", "Time", "TTime");

		searchBenchmark(1000, 3000, 50);
	}
}
